function Events(list, options) {

  let init = async function() {
    const targetJqueryObject = $(options.targetId);

    let textInfo = createTextInfoHTML();
    let inputList = await createListHTML();
    let addButton = await createAddButtonHTML();

    targetJqueryObject
      .empty()
      .append(textInfo)
      .append(inputList)
      .append(addButton);

    hideRemoveBtnOnFirstItem();
    $('.event-item-add-' + options.type).click(newItemClicked);
    $('.event-item-remove-' + options.type).click(removeItemClicked);
    $('.form-control').change(saveAllValues);
  }

  let hideRemoveBtnOnFirstItem = function() {
    $(options.targetId + ' .input-group').first().find('.event-item-remove-' + options.type).css('visibility', 'hidden');
  }

  let saveAllValues = function() {
    list.forEach(item => {
      let itemValue = $('#eventsItem-input-' + item.id).val() || -1;
      item.max_in_store_limit = itemValue === '' ? -1 : parseInt(itemValue);
    });
  }

  let removeItemClicked = function(event) {
    saveAllValues(); // First save all values
    let itemId = parseInt($(event.target).closest('button').attr('data-id'));
    let itemIndex = list.findIndex(item => item.id === itemId);
    list.splice(itemIndex, 1);
    init();
  }

  let newItemClicked = function() {
    saveAllValues(); // First save all values
    let maxId = list.reduce((a, b) => a.id > b.id ? a:b).id;
    list.push({
      id: ++maxId,
      type: options.type,
      max_in_store_limit: -1,
    });
    init();
  }

  let createTextInfoHTML = function() {
    return '<p class="ml-2 mr-2 mb-2 small text-left text-muted">' + options.text.info + '</p>';
  }

  let createAddButtonHTML = async function() {
    let addButtonHTML = await loadTemplate('events/addButtonTemplate.html');

    return addButtonHTML.replaceAll('%TYPE%', options.type);
  }

  let createListHTML = async function() {
    let listHTML = [];

    for (const item of list.filter(item => item.type === options.type)) {
      let itemHTML = await createItemHTML(item);
      listHTML.push(itemHTML);
    }

    return listHTML.join("");
  }

  let createItemHTML = async function(item) {
    let itemHTML = await loadTemplate('events/eventItemTemplate.html');
    let itemValue = item.max_in_store_limit;

    return itemHTML
      .replaceAll('%LABEL%', options.text.label)
      .replaceAll('%TYPE%', options.type)
      .replaceAll('%ITEM%', item.id)
      .replaceAll('%VALUE%', itemValue === -1 ? '' : itemValue);
  }

  let loadTemplate = async function(url) {
    try {
      return await $.get(url);
    } catch (e) {
      throw Error("Can't get template");
    }
  }

  return {
    init
  };

}
