var alertID = 0;
var propertiesResponse = null;

function getParameter(paramList) {
  var promise = new Promise(function(resolve, reject) {
    $.get("/axis-cgi/param.cgi?action=list&group=" + paramList, function(data) {
      resolve(parseParams(data));
    }).fail(function() {
      reject();
    });
  });
  return promise;
}

function setParameter(app_name, paramName, value) {
  var promise = new Promise(function(resolve, reject) {
    var dataJson = {};
    dataJson['action'] = 'update';
    dataJson[app_name + '.' + paramName] = value;

    $.ajax({
      method: "POST",
      cache: true,
      url: "/axis-cgi/param.cgi",
      data: dataJson
    }).done(function(msg) {
      try {
        if (msg.trim().toLowerCase() == 'ok') {
          resolve();
        } else {
          reject();
        }
      } catch(err) {
        reject();
      }

    });
  });
  return promise;
}

function getIOPortParameter(paramList) {
  var promise = new Promise(function(resolve, reject) {
    $.get("/axis-cgi/io/port.cgi?" + paramList, function(data) {
      resolve(parseParams(data));
    });
  });
  return promise;
}

function parseParams(data) {
  var params = {};
  var lines = data.split('\n');
  for (var i = 0; i < lines.length; i++) {
    var pos = lines[i].indexOf('=');
    if (pos != -1) {
      var name = lines[i].substring(0, pos).trim().toLowerCase();
      var value = lines[i].substring(pos + 1).trim();
      if (name.length) {
        if (value.length) {
          params[name] = value;
        } else {
          params[name] = "";
        }
      }
    }
  }
  return params;
}

function getService() {
  var promise = new Promise(function(resolve, reject) {
    $.ajax({
      method: "GET",
      cache: true,
      url: "/axis-cgi/param.cgi?action=list&group=StoreOccupancyManager.Configuration"
    }).done(function(data) {
      if (data) {
        resolve(data['StoreOccupancyManager.Configuration']);
      } else {
        reject();
      }
    }).fail(function() {
      reject();
    });
  });
  return promise;
}

function setService(enabled, service, counter_correction) {
  var promise = new Promise(function(resolve, reject) {
    var dataJson = {};
    dataJson['action'] = 'update';
    dataJson['StoreOccupancyManager.Enabled'] = enabled;
    dataJson['StoreOccupancyManager.Configuration'] = service;
    dataJson['StoreOccupancyManager.CounterCorrection'] = counter_correction;

    $.ajax({
      method: "POST",
      cache: true,
      url: "/axis-cgi/param.cgi",
      data: dataJson
    }).done(function(msg) {
      if (msg === 'OK') {
        resolve();
      } else {
        console.error(msg);
        reject();
      }
    });
  });
  return promise;
}

function isLicenseValid() {
  var promise = new Promise(function(resolve, reject) {
    $.get('/axis-cgi/applications/list.cgi').then(function (xmlResult) {
        let appInfo = $(xmlResult).find('reply').find('application[Name="storeoccupancymanager"]')[0];
        let licenseResult = $(appInfo).attr('License');
        let licenseExpiration = $(appInfo).attr('LicenseExpirationDate');
        licenseExpiration = licenseExpiration === undefined ? '' : licenseExpiration;
        resolve([licenseResult, licenseExpiration]);
      }, reject);
  });
  return promise;
}

function checkProxyCameraConnection(host, port, user, pass, type) {
  var promise = new Promise(function(resolve) {
    var url = '/local/storeoccupancymanager/api/proxy.cgi?';
    url += 'target_camera_ip=' + encodeURIComponent(host) + '&';
    url += 'target_camera_port=' + encodeURIComponent(port) + '&';
    let path = '/axis-cgi/param.cgi';
    if (type === 'xovis') {
      path = 'api/data/live?format=json';
    }
    url += 'target_camera_path=' + encodeURIComponent(path) + '&';
    url += 'target_camera_user=' + encodeURIComponent(user) + '&';
    url += 'target_camera_pass=' + encodeURIComponent(pass);
    if (type !== 'xovis') {
      url += '&action=list&group=Brand.Brand';
    }

    $.ajax({
      url: url,
      success: function() {
        resolve(['Connection successful.', 'success']);
      },
      error: function() {
        resolve(['Connection failed.', 'danger']);
      }
    });
  });
  return promise;
}

function getAvailableResolution(params) {
  let cameraResolutions = [];
  let resololutionsStr = params.split(',');
  for (var i = 0; i < resololutionsStr.length; i++) {
    var res = resololutionsStr[i].split('x');
    if (res.length > 1) {
      cameraResolutions.push({"width": parseInt(res[0].trim()), "height": parseInt(res[1].trim())});
    }
  }
  return cameraResolutions;
}

function sortFunc(a, b) {
  return b.width - a.width;
}

function showSaveWarning() {
  if (JSON.stringify(serviceArr) !== JSON.stringify(serviceArrTmp)) {
    const warningAlertName = 'saveWarning';
    const warningText = 'Unsaved changes';
    let html = '<div id="' + warningAlertName + '" class="alert alert-warning d-inline-flex" title="' + warningText + '">';
    html += '<strong><span class="fas fa-exclamation-triangle"></span></strong><span class="ml-2 d-none d-sm-block">' + warningText + '</span>';
    html += '</div>';
    $('#warningArea').html(html);
    if ($(window).width() < 769) { // Display tooltip only on mobile
      $('#' + warningAlertName).tooltip();
    }
  } else {
    hideSaveWarning();
  }
}

function validate(element, invalidValue, invalidMsg){
  var validation = (element.val() === invalidValue) ? 'is-invalid' : '';
  var invalidDiv = '<div class="invalid-feedback">' + invalidMsg + '</div>';

  if(validation === '') {
    element.removeClass('is-invalid');
    element.closest('.input-group').find('.invalid-feedback').remove();
  } else {
    element.addClass(validation);
    element.closest('.input-group').find('.invalid-feedback').remove();
    element.closest('.input-group').append(invalidDiv);
  }
}

$.fn.showSpinner = function(text = '') {
  var loadingDiv = '<div class="loadingDiv align-items-center align-items-center text-center row rounded h-100" style="position:absolute;top:0;bottom:0;left:15px;right:15px;background-color: #f7f7f7;z-index: 9999;">';
  loadingDiv += '<div class="col"><img src="../images/ajax-loader.gif" style="height: 30px; opacity: 0.2;" class="text-center" />';
  if (text !== '') loadingDiv += '<p class="text-muted mb-0">' + text + '</p>';
  loadingDiv += '</div></div>';
  $(this).append(loadingDiv);
};

$.fn.hideSpinner = function() {
  $(this).find('.loadingDiv').remove();
};

function hideSaveWarning() {
  $('#warningArea').html('');
}

function createResolutionList(selectID) {
  var selected = 0;
  var cameraResolution = -1;
  $(selectID).empty();
  var resItem = $(selectID)[0];
  for (var i = 0; i < cameraResolutions.length; i++) {
    if (cameraResolutions[i].width === serviceArrTmp.stream_width && cameraResolutions[i].height === serviceArrTmp.stream_height) {
      selected = i;
    }
    if (cameraResolutions[i].width === resolution.width && cameraResolutions[i].height === resolution.height) {
      cameraResolution = i;
    }
    resItem.options[resItem.options.length] = new Option(cameraResolutions[i].width + ' x ' + cameraResolutions[i].height, i);
  }
  resItem.selectedIndex = selected;
  if (cameraResolution != -1)
    $(selectID + ' option[value="' + cameraResolution + '"]').css('background-color', '#B6DAFF');
}

function checkHwIoState(triggeringElement) {
  let isHwInput = triggeringElement.val().match(/input\d+/) !== null;

  if (typeof timeOut === 'number') {
    clearTimeout(timeOut);
  }

  if (isHwInput) {
    let inputNumber = parseInt(triggeringElement.val().replace('input', '')) + 1;
    getIOPortParameter("checkactive="+inputNumber).then(function (responseData) {
      if(responseData['port' + inputNumber] === "active") {
        $('.hw_badge.hw_badge_off').hide();
        $('.hw_badge.hw_badge_on').show();
      } else {
        $('.hw_badge.hw_badge_on').hide();
        $('.hw_badge.hw_badge_off').show();
      }
      timeOut = setTimeout(checkHwIoState, 5000, triggeringElement);
    });
  } else {
    $('.hw_badge').hide();
  }
}

function createVideoChannelElement(selectID, selectedVal) {
  var promise = new Promise(function(resolve, reject) {
    var selectedVideoChannel = createVideoChannelList(selectID, selectedVal, nbrOfConfigs, params); // Return array of selected view area
    for(var i = 0; i < selectedVideoChannel.length; i++) {
      $(selectID + ' [data-id="' + selectedVideoChannel[i] + '"]').prop('checked', true);
    }
    resolve();
  });
  return promise;
}

function setNewDefaultResolution(resolutionProperty) {
  var res = resolutionProperty.split('x');
  if (res.length > 1) {
    resolution = {"width": parseInt(res[0].trim()), "height": parseInt(res[1].trim())};
  }
}

function setNewResolutionList(resolutionProperties) {
  if (resolutionProperties === undefined) {
    resolutionProperties = params['properties.image.resolution'];
  }
  cameraResolutions = getAvailableResolution(resolutionProperties);
  cameraResolutions = cameraResolutions.sort(sortFunc);
}

function createCameraPreview(element, width, height, videoChannel) {
  videoChannel = videoChannel.length === 0 ? '' : Math.min.apply(Math, videoChannel);
  showImageFromCamera(width, height, videoChannel);
}

function createVideoChannelList(selectID, selectedVal, videoChannelCount, cameraProperties) {
  var selected = [];
  var element = $(selectID);
  element.empty();
  for (var i = 0; i < videoChannelCount; i++) {
    if (cameraProperties['image.i' + i + '.enabled'] === 'yes') {
      if(selectedVal.includes(i)) {
        selected.push(i);
      }
      element.append('<div class="form-check float-left ml-2"><input type="checkbox" class="videoChannel form-check-input" id="videoChannel' + i + '" data-id="' + i + '"><label for="videoChannel' + i + '" class="form-check-label">' + cameraProperties['image.i' + i + '.name'] + '</label></div>')
    }
  }
  return selected;
}

function createVisibilityList(selectId, selectedVal) {
  var vItem = $(selectId);
  vItem.empty();
  vItem.append(new Option('Always', 'always'));
  vItem.append(new Option('Entry allowed', 'enter'));
  vItem.append(new Option('No entry', 'stop'));
  vItem.val(selectedVal);
}

function createPositionList(selectID, selectedVal) {
  $(selectID).html('');
  var posItem = $(selectID);
  posItem.empty();
  posItem.append(new Option('Top Left', 'top_left'));
  posItem.append(new Option('Top Center', 'top'));
  posItem.append(new Option('Top Right', 'top_right'));
  posItem.append(new Option('Left Center', 'left'));
  posItem.append(new Option('Center', 'center'));
  posItem.append(new Option('Right Center', 'right'));
  posItem.append(new Option('Bottom Left', 'bottom_left'));
  posItem.append(new Option('Bottom Center', 'bottom'));
  posItem.append(new Option('Bottom Right', 'bottom_right'));
  posItem.val(selectedVal);
}

function getImageList() {
  var promise = new Promise(function(resolve, reject) {
    $.get('/local/storeoccupancymanager/api/upload_image.cgi?action=list', function(response) {
        if (response.code == 200 && response.list != undefined) {
          var list = response.list.sort(function(a, b) {
            if (a.name < b.name) {
              return -1;
            }
            if (a.name > b.name) {
              return 1;
            }
            return 0;
          });
          resolve(list);
        } else {
          resolve([]);
        }
    });
  });
  return promise;
}

/** Attach validation to elements in form
 *
 * @param options - configuration array
 */
$.fn.attachValidation = function(options) {
  const element = $(this); //input element

  validatePattern(element, options); //validate on start

  element.on('keyup', function() { //validation on keyup event
    validatePattern(element, options);
  });
};

/** Validation with pattern
 *
 * @param elem - jQuery wrapper $(this)
 * @param options - configuration array
 */
function validatePattern(elem, options) {
  const element = elem;
  let pattern = options.pattern || element.attr('pattern');
  let regex = RegExp(pattern);
  let pattern_msg = options.invalidMsg || element.attr('data-pattern-title');
  let valid = regex.test(element.val());
  showValidationResult(element, valid, pattern_msg, options.disableBtnId);
}

/** Validate element with range
 *
 * @param jqueryElement - jQuery wrapper $(this)
 * @param options - configuration array
 */
function validateRange(jqueryElement, options) {
  const elementToValidate = jqueryElement;
  const value = parseInt(elementToValidate.val());
  let validResult = false;
  if (!isNaN(value) && options.minVal <= value && value <= options.maxVal) {
    validResult = true;
  }
  showValidationResult(elementToValidate, validResult, options.invalidMsg, options.disableBtnId);
}

/** Show validation result
 *
 * @param element - jQuery wrapper $(this)
 * @param valid - regex result if is valid
 * @param invalidMsg - error msg displayed in body
 * @param disableBtnId - id of btn to disable
 */
function showValidationResult(element, valid, invalidMsg, disableBtnId){
  let validation = valid ? '' : 'is-invalid';
  const invalidDiv = '<div class="invalid-feedback text-right mr-3">' + invalidMsg + '</div>';

  if(validation === '') {
    element.removeClass('is-invalid');
    element.closest('.input-group').find('.invalid-feedback').remove();
  } else {
    element.addClass(validation);
    element.closest('.input-group').find('.invalid-feedback').remove();
    element.closest('.input-group').append(invalidDiv);
    element.closest('.input-group').find('.invalid-feedback').show();
  }
  const element_button_to_disable = $('#' + disableBtnId);
  const invalid_elements_in_form = element.closest('form').find('.is-invalid').length > 0;
  element_button_to_disable.prop('disabled', invalid_elements_in_form);
}

/**
 * Create HTML options for select
 * @param {string} jquerySelector
 * @param {boolean} allowUrl
 * @param {string} notSelectedValueLabel
 * @param {string} selectedVal
 * @returns {Promise<void>}
 */
async function createImageListInOption(jquerySelector, allowUrl, notSelectedValueLabel, selectedVal) {
  let imageList = await imageManager.getFileList();
  const allowGif = !isMips();

  $(jquerySelector)
    .empty()
    .append(new Option(notSelectedValueLabel, '', false, false));

  imageList.forEach(item => {
    let path = item.path;
    let isUrl = (item.storage === "url");
    let isGif = path.includes('.gif');
    if (!allowUrl) {
      //in customGraphics, controlRoom is not valid whole url, only without file://
      path = path.replace('file://', '');
    }
    if ((allowUrl || !isUrl) && (allowGif || !isGif)) {
      $(jquerySelector).append(new Option(item.name, path, false, false));
    }
  });

  $(jquerySelector).val(selectedVal);
}

function convertScaleFloatToPercentage(value) {
  return Math.round(value * 100);
}

function convertScalePercentageToFloat(value) {
  if (value === "") {
    value = 100;
  }
  return parseFloat(value) / 100;
}

function hexToRgb(hex){
  var c;
  if(/^#([A-Fa-f0-9]{3}){1,2}$/.test(hex)){
    c= hex.substring(1).split('');
    if(c.length== 3){
      c= [c[0], c[0], c[1], c[1], c[2], c[2]];
    }
    c= '0x'+c.join('');
    return [(c>>16)&255, (c>>8)&255, c&255].join(',')+',1';
  }else{
    throw new Error('bad HEX input');
  }
}

function rgbToHex(rgb){
  rgb = rgb.match(/^[\s+]?[\s+]?(\d+)[\s+]?,[\s+]?(\d+)[\s+]?,[\s+]?(\d+)[\s+]?/i);
  if(rgb && rgb.length === 4){
    var hex =  "#" + ("0" + parseInt(rgb[1],10).toString(16)).slice(-2) +
      ("0" + parseInt(rgb[2],10).toString(16)).slice(-2) +
      ("0" + parseInt(rgb[3],10).toString(16)).slice(-2);
    return hex;
  }else{
    throw new Error('bad RGBA input');
  }
}

function escapeHtml(html) {
  return html.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;");
}